Package: PAA
Version: 1.45.0
Title: PAA (Protein Array Analyzer)
Authors@R: c(person("Michael", "Turewicz", role=c("aut", "cre"),
		    email="michael.turewicz@rub.de"),
	     person("Martin", "Eisenacher", role=c("ctb", "cre"),
		    email="martin.eisenacher@rub.de"))
Author: Michael Turewicz [aut, cre], Martin Eisenacher [ctb, cre]
Maintainer: Michael Turewicz <michael.turewicz@rub.de>, Martin Eisenacher <martin.eisenacher@rub.de>
Depends: R (>= 3.2.0), Rcpp (>= 0.11.6)
Imports: e1071, gplots, gtools, limma, MASS, mRMRe, randomForest, ROCR,
        sva
LinkingTo: Rcpp
Suggests: BiocStyle, RUnit, BiocGenerics, vsn
Description: PAA imports single color (protein) microarray data that has been saved in gpr 
             file format - esp. ProtoArray data. After preprocessing (background correction, 
             batch filtering, normalization) univariate feature preselection is performed 
             (e.g., using the "minimum M statistic" approach - hereinafter referred to as "mMs"). 
             Subsequently, a multivariate feature selection is conducted to discover biomarker 
             candidates. Therefore, either a frequency-based backwards elimination aproach or 
             ensemble feature selection can be used. PAA provides a complete toolbox of analysis 
             tools including several different plots for results examination and evaluation. 
License: BSD_3_clause + file LICENSE
URL: http://www.ruhr-uni-bochum.de/mpc/software/PAA/
SystemRequirements: C++ software package Random Jungle
biocViews: Classification, Microarray, OneChannel, Proteomics
git_url: https://git.bioconductor.org/packages/PAA
git_branch: devel
git_last_commit: dfb636c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Packaged: 2025-10-31 23:20:53 UTC; biocbuild
