# This file was generated by Rcpp::compileAttributes
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

joinMCountResults <- function(p1, p2, m1, m2, l1, l2) {
    .Call('PAA_joinMCountResults', PACKAGE = 'PAA', p1, p2, m1, m2, l1, l2)
}

mCount <- function(x, y, z, a, b) {
    .Call('PAA_mCount', PACKAGE = 'PAA', x, y, z, a, b)
}

mMsMatrix <- function(x, y) {
    .Call('PAA_mMsMatrix', PACKAGE = 'PAA', x, y)
}

sampling <- function(x, c, c1, c2, tc1, tc2) {
    .Call('PAA_sampling', PACKAGE = 'PAA', x, c, c1, c2, tc1, tc2)
}

